@@ -1,19 +1,19 @@
 // Copyright (C) 1999-2005 Open Source Telecom Corporation.
-//  
+//
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
 // the Free Software Foundation; either version 2 of the License, or
 // (at your option) any later version.
-// 
+//
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
 // You should have received a copy of the GNU General Public License
-// along with this program; if not, write to the Free Software 
+// along with this program; if not, write to the Free Software
 // Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
-// 
+//
 // As a special exception, you may use this file as part of a free software
 // library without restriction.  Specifically, if other files instantiate
 // templates or use macros or inline functions from this file, or you compile
@@ -103,7 +103,7 @@ const char *ScriptChecks::chkCall(Line *line, ScriptImage *img)
 		++cp;
 
 	if(strchr(cp, ':') || stricmp(line->cmd, "call"))
-	{		
+	{
 		snprintf(proto, sizeof(proto), "keywords.%s", cp);
 		list = (Line *)img->getPointer(proto);
 	}
@@ -260,13 +260,13 @@ const char *ScriptChecks::chkReturn(Line *line, ScriptImage *img)
 
 	if(cp)
 		return "member not used in this command";
-	
+
 	return NULL;
 }
-	
+
 const char *ScriptChecks::chkIgnore(Line *line, ScriptImage *img)
 {
-        return NULL;
+	return NULL;
 }
 
 const char *ScriptChecks::chkDecimal(Line *line, ScriptImage *img)
@@ -323,22 +323,22 @@ const char *ScriptChecks::chkSlog(Line *line, ScriptImage *img)
 
 	if(member)
 	{
-                if(!stricmp(member, ".debug"))
-                        member = NULL;
-                else if(!stricmp(member, ".info"))
-                        member = NULL;
-                else if(!stricmp(member, ".notice"))
-                        member = NULL;
-                else if(!strnicmp(member, ".warn", 5))
-                        member = NULL;
-                else if(!strnicmp(member, ".err", 4))
-                        member = NULL;
-                else if(!strnicmp(member, ".crit", 5))
-                        member = NULL;
-                else if(!stricmp(member, ".alert"))
+		if(!stricmp(member, ".debug"))
+			member = NULL;
+		else if(!stricmp(member, ".info"))
+			member = NULL;
+		else if(!stricmp(member, ".notice"))
+			member = NULL;
+		else if(!strnicmp(member, ".warn", 5))
+			member = NULL;
+		else if(!strnicmp(member, ".err", 4))
+			member = NULL;
+		else if(!strnicmp(member, ".crit", 5))
+			member = NULL;
+		else if(!stricmp(member, ".alert"))
+			member = NULL;
+		else if(!strnicmp(member, ".emerg", 6))
 			member = NULL;
-                else if(!strnicmp(member, ".emerg", 6))
-                        member = NULL;
 	}
 
 	if(member)
@@ -349,10 +349,10 @@ const char *ScriptChecks::chkSlog(Line *line, ScriptImage *img)
 
 const char *ScriptChecks::chkHasArgs(Line *line, ScriptImage *img)
 {
-        if(!line->argc)
-                return "arguments missing";
+	if(!line->argc)
+		return "arguments missing";
 
-        return NULL;
+	return NULL;
 }
 
 const char *ScriptChecks::chkOnlyArgs(Line *line, ScriptImage *img)
@@ -364,7 +364,7 @@ const char *ScriptChecks::chkOnlyArgs(Line *line, ScriptImage *img)
 		return "keywords not used for this command";
 
 	return NULL;
-}	
+}
 
 const char *ScriptChecks::chkOnlyOneArg(Line *line, ScriptImage *img)
 {
@@ -401,11 +401,11 @@ const char *ScriptChecks::chkIndex(Line *line, ScriptImage *img)
 
 const char *ScriptChecks::chkExpr(Line *line, ScriptImage *img)
 {
-        const char *cp = getMember(line);
+	const char *cp = getMember(line);
 
-        if(cp && !isdigit(cp[0]))
+	if(cp && !isdigit(cp[0]))
 	{
-                cp = chkProperty(line, img);
+		cp = chkProperty(line, img);
 
 		if(cp)
 			return cp;
@@ -446,11 +446,11 @@ const char *ScriptChecks::chkString(Line *line, ScriptImage *img)
 
 const char *ScriptChecks::chkDefine(Line *line, ScriptImage *img)
 {
-        unsigned idx = 0;
-        const char *cp = line->cmd;
+	unsigned idx = 0;
+	const char *cp = line->cmd;
 
-        if(!line->argc)
-                return "define requires arguments";
+	if(!line->argc)
+		return "define requires arguments";
 
 	while(idx < line->argc)
 	{
@@ -480,7 +480,7 @@ const char *ScriptChecks::chkDefine(Line *line, ScriptImage *img)
 }
 
 const char *ScriptChecks::chkVarType(Line *line, ScriptImage *img)
-{  
+{
 	if(getMember(line))
 		return "no members in type";
 
@@ -489,24 +489,24 @@ const char *ScriptChecks::chkVarType(Line *line, ScriptImage *img)
 
 	return chkAllVars(line, img);
 }
-	
-const char *ScriptChecks::chkVar(Line *line, ScriptImage *img)    
-{
-        const char *cp = getMember(line);
 
-        if(cp && !isdigit(cp[0]))
+const char *ScriptChecks::chkVar(Line *line, ScriptImage *img)
+{
+	const char *cp = getMember(line);
+
+	if(cp && !isdigit(cp[0]))
 	{
-                cp = chkProperty(line, img);
+		cp = chkProperty(line, img);
 		if(!cp)
 			return "property invalid for var";
 	}
-        else
-                cp = NULL;
+	else
+		cp = NULL;
 
-        if(!useKeywords(line, "=size=value"))
-                return "invalid keyword used";
+	if(!useKeywords(line, "=size=value"))
+		return "invalid keyword used";
 
-        return chkAllVars(line, img);
+	return chkAllVars(line, img);
 }
 
 
@@ -526,10 +526,34 @@ const char *ScriptChecks::chkNumber(Line *line, ScriptImage *img)
 	return chkAllVars(line, img);
 }
 
+const char *ScriptChecks::chkConstruct(Line *line, ScriptImage *img)
+{
+	unsigned int idx = 0;
+	const char *cp = getOption(line, &idx);
+	if(getMember(line))
+		return "no members for this command";
+
+	if(!cp)
+		return "destination for contruct is missing";
+
+	if(*cp != '%' && *cp != '&')
+		return "destination of contruct must be symbol";
+
+	if(getOption(line,&idx))
+		return "only one target for contruct";
+
+	return NULL;
+}
+
 const char *ScriptChecks::chkPack(Line *line, ScriptImage *img)
 {
-	if(getMember(line))
-		return "member not used for this command";
+	const char *cp = getMember(line);
+	if(cp && stricmp(cp, "struct"))
+		return "only .struct may be used for this command";
+
+	if(!stricmp(cp, "struct"))
+		if(!useKeywords(line, "=size"))
+			return "invalid keyword used for pack.struct";
 
 	if(!useKeywords(line, "=field=offset=token=size=quote=prefix=suffix"))
 		return "invalid keyword used for this command";
@@ -569,7 +593,7 @@ const char *ScriptChecks::chkExpression(Line *line, ScriptImage *img)
 		return "unbalanced parenthesis in expression";
 
 	return NULL;
-}	
+}
 
 const char *ScriptChecks::chkAllVars(Line *line, ScriptImage *img)
 {
@@ -601,7 +625,7 @@ const char *ScriptChecks::chkType(Line *line, ScriptImage *img)
 		return "type does not use keyword";
 
 	if(line->argc < 1)
-		return "type requires at least one var";	
+		return "type requires at least one var";
 
 	return chkAllVars(line, img);
 }
@@ -652,7 +676,7 @@ const char *ScriptChecks::chkSet(Line *line, ScriptImage *img)
 	if(!useKeywords(line, "=size=offset"))
 		return "invalid keyword used";
 
-	return chkFirstVar(line, img);	
+	return chkFirstVar(line, img);
 }
 
 const char *ScriptChecks::chkFor(Line *line, ScriptImage *img)
@@ -671,8 +695,8 @@ const char *ScriptChecks::chkForeach(Line *line, ScriptImage *img)
 	if(getMember(line))
 		return "member not used for this command";
 
-        if(!useKeywords(line, "=index=size=token"))
-                return "invalid keyword used";
+	if(!useKeywords(line, "=index=size=token"))
+		return "invalid keyword used";
 
 	if(getCount(line) != 2)
 		return "incorrect number of arguments";
@@ -691,15 +715,15 @@ const char *ScriptChecks::chkCat(Line *line, ScriptImage *img)
 	return chkFirstVar(line, img);
 }
 
-const char *ScriptChecks::chkRemove(Line *line, ScriptImage *img)     
+const char *ScriptChecks::chkRemove(Line *line, ScriptImage *img)
 {
-        if(getMember(line))  
-                return "member not used for this command";  
+	if(getMember(line))
+		return "member not used for this command";
 
-        if(!useKeywords(line, "=field=token=offset"))
-                return "invalid keyword used for this command";
+	if(!useKeywords(line, "=field=token=offset"))
+		return "invalid keyword used for this command";
 
-        return chkFirstVar(line, img);
+	return chkFirstVar(line, img);
 }
 
 const char *ScriptChecks::chkSequence(Line *line, ScriptImage *img)
@@ -772,15 +796,15 @@ const char *ScriptChecks::chkRefArgs(Line *line, ScriptImage *img)
 
 const char *ScriptChecks::chkProperty(Line *line, ScriptImage *img)
 {
-        const char *cp = getMember(line);
-        ScriptProperty *prop;                        
+	const char *cp = getMember(line);
+	ScriptProperty *prop;
 
-        if(!cp) 
-                return NULL;
+	if(!cp)
+		return NULL;
 
-        prop = ScriptProperty::find(cp);
-        if(!prop)
-                return "unknown script property referenced";
+	prop = ScriptProperty::find(cp);
+	if(!prop)
+		return "unknown script property referenced";
 
 	return NULL;
 }
