@@ -1,19 +1,19 @@
 // Copyright (C) 1999-2005 Open Source Telecom Corporation.
-//  
+//
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
 // the Free Software Foundation; either version 2 of the License, or
 // (at your option) any later version.
-// 
+//
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
 // You should have received a copy of the GNU General Public License
-// along with this program; if not, write to the Free Software 
+// along with this program; if not, write to the Free Software
 // Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
-// 
+//
 // As a special exception, you may use this file as part of a free software
 // library without restriction.  Specifically, if other files instantiate
 // templates or use macros or inline functions from this file, or you compile
@@ -66,7 +66,7 @@ long ScriptInterp::getRealValue(double d, unsigned prec)
 	return rval + atol(lval);
 }
 
-long ScriptInterp::getIntValue(const char *text, unsigned prec, ScriptProperty *p) 
+long ScriptInterp::getIntValue(const char *text, unsigned prec, ScriptProperty *p)
 {
 	Fun *fun = ifun;
 	unsigned count;
@@ -105,7 +105,7 @@ long ScriptInterp::getIntValue(const char *text, unsigned prec, ScriptProperty *
 			return rval - atol(lval);
 
 		return rval + atol(lval);
-	}	
+	}
 
 	while(NULL != fun)
 	{
@@ -133,14 +133,14 @@ long ScriptInterp::getIntValue(const char *text, unsigned prec, ScriptProperty *
 	if(count != fun->args)
 		return 0;
 
-	rval = fun->fn(ival, prec);	
+	rval = fun->fn(ival, prec);
 	delete[] ival;
 	return rval;
-}	
+}
 
 double ScriptInterp::getDouble(long value, unsigned prec)
 {
-        return (double)value / (double)tens[prec];
+	return (double)value / (double)tens[prec];
 }
 
 long ScriptInterp::getInteger(long value, unsigned prec)
@@ -240,7 +240,7 @@ int ScriptInterp::numericExpression(long *vals, int max, unsigned prec, ScriptPr
 			}
 			continue;
 		}
-	
+
 		if(!*expr)
 		{
 			vstack[esp] = getIntValue(value, prec, p);
@@ -351,12 +351,12 @@ bool ScriptInterp::conditionalExpression(void)
 		op = v1;
 	else if(!stricmp(v1, "-array") || !stricmp(v1, "!array"))
 		op = v1;
-        else if(!stricmp(v1, "-queue") || !stricmp(v1, "!queue"))
-                op = v1;
+	else if(!stricmp(v1, "-queue") || !stricmp(v1, "!queue"))
+		op = v1;
 	else if(!stricmp(v1, "-empty") || !stricmp(v1, "!empty"))
 		op = v1;
-        else if(!stricmp(v1, "-active") || !stricmp(v1, "!active"))
-                op = v1;
+	else if(!stricmp(v1, "-active") || !stricmp(v1, "!active"))
+		op = v1;
 	else if(!stricmp(v1, "-module") || !stricmp(v1, "-command") || !stricmp(v1, "-has"))
 		op = "-module";
 	else if(!stricmp(v1, "!module") || !stricmp(v1, "!command"))
@@ -409,7 +409,7 @@ bool ScriptInterp::conditionalExpression(void)
 	{
 		if(!getScript(getContent(v2)))
 			return true;
-	
+
 		return false;
 	}
 
@@ -494,60 +494,60 @@ bool ScriptInterp::conditionalExpression(void)
 		}
 	}
 
-        if(!stricmp(op, "!modify") || !stricmp(op, "-const"))
-        {
-                sym = mapSymbol(v2, 0);
-                if(!sym)
-                        return true;
+	if(!stricmp(op, "!modify") || !stricmp(op, "-const"))
+	{
+		sym = mapSymbol(v2, 0);
+		if(!sym)
+			return true;
 
-                switch(sym->type)
-                {
-                case symLOCK:  
-                case symCONST:
-                case symSEQUENCE:
-                        return true;
+		switch(sym->type)
+		{
+				case symLOCK:
+				case symCONST:
+				case symSEQUENCE:
+			return true;
 		default:
 			return false;
-                }
-        }  
+		}
+	}
 
-        if(!stricmp(op, "-queue"))
-        {
-                sym = mapSymbol(v2, 0);
-                if(!sym)
-                        return false;
+	if(!stricmp(op, "-queue"))
+	{
+		sym = mapSymbol(v2, 0);
+		if(!sym)
+			return false;
 
-                switch(sym->type)
-                {
-                case symSTACK:
+		switch(sym->type)
+		{
+				case symSTACK:
 		case symFIFO:
-                        return true;
+			return true;
 		default:
 			return false;
-                }
-        }
+		}
+	}
 
-        if(!stricmp(op, "!queue"))
-        {
-                sym = mapSymbol(v2, 0);
-                if(!sym)
-                        return false;
+	if(!stricmp(op, "!queue"))
+	{
+		sym = mapSymbol(v2, 0);
+		if(!sym)
+			return false;
 
-                switch(sym->type)
-                {
-                case symFIFO:
+		switch(sym->type)
+		{
+				case symFIFO:
 		case symSTACK:
-                        return false;
+			return false;
 		default:
 			return true;
-                }
-        }
+		}
+	}
 
 	if(!stricmp(op, "-array"))
 	{
-                sym = mapSymbol(v2, 0);
-                if(!sym)
-                        return false;
+		sym = mapSymbol(v2, 0);
+		if(!sym)
+			return false;
 
 		switch(sym->type)
 		{
@@ -558,21 +558,21 @@ bool ScriptInterp::conditionalExpression(void)
 		}
 	}
 
-        if(!stricmp(op, "!array"))  
-        {
-                sym = mapSymbol(v2, 0);
-                if(!sym)
-                        return true;
+	if(!stricmp(op, "!array"))
+	{
+		sym = mapSymbol(v2, 0);
+		if(!sym)
+			return true;
 
-                switch(sym->type)
-                {
-                case symARRAY:
-                        return false;
+		switch(sym->type)
+		{
+				case symARRAY:
+			return false;
 		default:
 			return true;
-                }
-        }   
-   
+		}
+	}
+
 	if(!stricmp(op, "-defined"))
 	{
 		sym = mapSymbol(v2, 0);
@@ -742,6 +742,39 @@ bool ScriptInterp::conditionalExpression(void)
 		return false;
 	}
 
+	if(!stricmp(op, ".of.") || !stricmp(op, "?"))
+	{
+		l1 = strlen(v1);
+		while(v2 && *v2)
+		{
+			if(!strnicmp(v1, v2, l1))
+			{
+				if(v2[l1] == '=' || v2[l1] == ':')
+					return true;
+			}
+			v2 = strchr(v2, ';');
+			if(v2)
+				++v2;
+		}
+		return false;
+	}
+
+	if(!stricmp(op, "!?"))
+	{
+		l1 = strlen(v1);
+		while(v2 && *v2)
+		{
+			if(!strnicmp(v1, v2, l1))
+			{
+				if(v2[l1] == '=' || v2[l1] == ':')
+					return false;
+			}
+			v2 = strchr(v2, ';');
+			if(v2)
+				++v2;
+		}
+		return true;
+	}
 
 	if(!stricmp(op, "$") || !stricmp(op, ".in."))
 	{
