@@ -1,19 +1,19 @@
 // Copyright (C) 1999-2005 Open Source Telecom Corporation.
-//  
+//
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
 // the Free Software Foundation; either version 2 of the License, or
 // (at your option) any later version.
-// 
+//
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
 // You should have received a copy of the GNU General Public License
-// along with this program; if not, write to the Free Software 
+// along with this program; if not, write to the Free Software
 // Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
-// 
+//
 // As a special exception, you may use this file as part of a free software
 // library without restriction.  Specifically, if other files instantiate
 // templates or use macros or inline functions from this file, or you compile
@@ -48,9 +48,9 @@ bool ScriptMethods::scrNop(void)
 	return true;
 }
 
-bool ScriptMethods::scrIf(void)   
+bool ScriptMethods::scrIf(void)
 {
-        if(conditional())
+	if(conditional())
 	{
 		if(frame[stack].index < frame[stack].line->argc)
 			return intGoto();
@@ -60,16 +60,16 @@ bool ScriptMethods::scrIf(void)
 		return true;
 	}
 	skip();
-        return true;
+	return true;
 }
 
 bool ScriptMethods::scrIfThen(void)
 {
-        if(!conditional())
-                skip();
+	if(!conditional())
+		skip();
 
-        skip();
-        return true;
+	skip();
+	return true;
 }
 
 bool ScriptMethods::scrSlog(void)
@@ -77,29 +77,29 @@ bool ScriptMethods::scrSlog(void)
 	const char *member = getMember();
 	const char *opt;
 
-        if(member)
-        {
-                if(!stricmp(member, "debug"))
-                        slog(Slog::levelDebug);
-                else if(!stricmp(member, "info"))
-                        slog(Slog::levelInfo);
-                else if(!stricmp(member, "notice"))
-                        slog(Slog::levelNotice);
-                else if(!strnicmp(member, "warn", 4))
-                        slog(Slog::levelWarning);
-                else if(!strnicmp(member, "err", 3))
-                        slog(Slog::levelError);
-                else if(!strnicmp(member, "crit", 4))
-                        slog(Slog::levelCritical);
-                else if(!stricmp(member, "alert"))
-                        slog(Slog::levelAlert);
-                else if(!strnicmp(member, "emerg", 5))
-                        slog(Slog::levelEmergency);
-                else
-                        slog(Slog::levelNotice);
-        }
-        else
-                slog(Slog::levelNotice);
+	if(member)
+	{
+		if(!stricmp(member, "debug"))
+			slog(Slog::levelDebug);
+		else if(!stricmp(member, "info"))
+			slog(Slog::levelInfo);
+		else if(!stricmp(member, "notice"))
+			slog(Slog::levelNotice);
+		else if(!strnicmp(member, "warn", 4))
+			slog(Slog::levelWarning);
+		else if(!strnicmp(member, "err", 3))
+			slog(Slog::levelError);
+		else if(!strnicmp(member, "crit", 4))
+			slog(Slog::levelCritical);
+		else if(!stricmp(member, "alert"))
+			slog(Slog::levelAlert);
+		else if(!strnicmp(member, "emerg", 5))
+			slog(Slog::levelEmergency);
+		else
+			slog(Slog::levelNotice);
+	}
+	else
+		slog(Slog::levelNotice);
 
 	slog() << logname << ": ";
 	while(NULL != (opt = getValue(NULL)))
@@ -111,59 +111,59 @@ bool ScriptMethods::scrSlog(void)
 
 bool ScriptMethods::scrThen(void)
 {
-        int level = 0;
-        Line *line;
+	int level = 0;
+	Line *line;
 
-        skip();
+	skip();
 
-        while(NULL != (line = frame[stack].line))
-        {
-                skip();
+	while(NULL != (line = frame[stack].line))
+	{
+		skip();
 
-                if(line->scr.method == (Method)&ScriptMethods::scrThen)
-                        ++level;
-                else if(line->scr.method == (Method)&ScriptMethods::scrElse)
-                {   
-                        if(!level)
-                                return true;
-                }
-                else if(line->scr.method == (Method)&ScriptMethods::scrEndif)
-                {
-                        if(!level)
-                                return true;
-                        --level;
-                }   
-        }
+		if(line->scr.method == (Method)&ScriptMethods::scrThen)
+			++level;
+		else if(line->scr.method == (Method)&ScriptMethods::scrElse)
+		{
+			if(!level)
+				return true;
+		}
+		else if(line->scr.method == (Method)&ScriptMethods::scrEndif)
+		{
+			if(!level)
+				return true;
+			--level;
+		}
+	}
 	return true;
 }
 
 bool ScriptMethods::scrElse(void)
 {
-        int level = 0;
-        Line *line;
+	int level = 0;
+	Line *line;
 
 	skip();
 
-        while(NULL != (line = frame[stack].line))
-        {
+	while(NULL != (line = frame[stack].line))
+	{
 		skip();
 
-                if(line->scr.method == (Method)&ScriptMethods::scrThen)
-                        ++level;
-                else if(line->scr.method == (Method)&ScriptMethods::scrEndif)
-                {
-                        if(!level)
-                                return true;
-                }
-        }
+		if(line->scr.method == (Method)&ScriptMethods::scrThen)
+			++level;
+		else if(line->scr.method == (Method)&ScriptMethods::scrEndif)
+		{
+			if(!level)
+				return true;
+		}
+	}
 	return true;
 }
 
 bool ScriptMethods::scrEndif(void)
 {
-        frame[stack].tranflag = false;
-        skip();
-        return true;
+	frame[stack].tranflag = false;
+	skip();
+	return true;
 }
 
 bool ScriptMethods::scrInit(void)
@@ -181,7 +181,7 @@ bool ScriptMethods::scrInit(void)
 		cp = line->args[idx++];
 		if(*cp != '=')
 			continue;
-		
+
 		value = getContent(line->args[idx++]);
 		syms = getLocal();
 		sym = syms->find(++cp, (unsigned short)strlen(value));
@@ -229,7 +229,7 @@ bool ScriptMethods::scrRemove(void)
 	Line *line = getLine();
 	bool queue = false;
 	unsigned offset = 0, len, count;
-	
+
 	opt = getKeyword("offset");
 	if(!opt)
 		opt = getKeyword("field");
@@ -252,9 +252,9 @@ bool ScriptMethods::scrRemove(void)
 		return true;
 	}
 
-        buffer[0] = 0;
-        while(NULL != (opt = getValue(NULL)))
-                addString(buffer, sizeof(buffer), opt);
+	buffer[0] = 0;
+	while(NULL != (opt = getValue(NULL)))
+		addString(buffer, sizeof(buffer), opt);
 	key = opt = buffer;
 	count = offset;
 	while(key && count--)
@@ -269,7 +269,7 @@ bool ScriptMethods::scrRemove(void)
 	count = 0;
 	while(key[count] && key[count] != pack)
 		++count;
-	
+
 	a = (Array *)&sym->data;
 
 	switch(sym->type)
@@ -372,11 +372,11 @@ bool ScriptMethods::scrRemove(void)
 		}
 		if(a->tail)
 			--a->tail;
-		break;	
+		break;
 	case symPROPERTY:
 		memcpy(&p, &sym->data, sizeof(p));
 		if(p->token())
-			pack = p->token();			
+			pack = p->token();
 	case symNORMAL:
 	case symNUMBER:
 		if(queue)
@@ -405,7 +405,7 @@ bool ScriptMethods::scrRemove(void)
 			++cp;
 		}
 		*cp = 0;
-		break;								
+		break;
 	default:
 		error("symbol-invalid");
 		return true;
@@ -513,7 +513,7 @@ bool ScriptMethods::scrSet(void)
 	const char *opt = getMember();
 	ScriptProperty *prop = ScriptProperty::find(opt);
 	unsigned short size = symsize;
-	Symbol *sym;	
+	Symbol *sym;
 	Line *line = getLine();
 	bool cat = false;
 	bool rtn;
@@ -551,7 +551,7 @@ bool ScriptMethods::scrSet(void)
 		error("set-invalid-symbol");
 		return true;
 	}
-	
+
 	buffer[0] = 0;
 	while(NULL != (opt = getValue(NULL)))
 		addString(buffer, max + 1, opt);
@@ -577,7 +577,7 @@ bool ScriptMethods::scrSet(void)
 			skip();
 			return true;
 		}
-	}	
+	}
 
 	if(!sym)
 	{
@@ -644,7 +644,7 @@ bool ScriptMethods::scrConst(void)
 		error("const-invalid-symbol");
 		return true;
 	}
-	
+
 	buffer[0] = 0;
 	while(NULL != (opt = getValue(NULL)))
 		addString(buffer, max + 1, opt);
@@ -661,7 +661,7 @@ bool ScriptMethods::scrConst(void)
 	}
 	else
 		opt = buffer;
-	
+
 	if(setConst(id, opt))
 		skip();
 	else
@@ -718,9 +718,9 @@ bool ScriptMethods::scrSequence(void)
 
 bool ScriptMethods::scrDecimal(void)
 {
-        frame[stack].decimal = atoi(getValue("0"));
-        skip();
-        return true;
+	frame[stack].decimal = atoi(getValue("0"));
+	skip();
+	return true;
 }
 
 bool ScriptMethods::scrError(void)
@@ -731,18 +731,18 @@ bool ScriptMethods::scrError(void)
 
 	while(NULL != (cp = getValue(NULL)))
 		addString(buffer, sizeof(buffer), cp);
-	
+
 	error(buffer);
 	return true;
 }
 
 bool ScriptMethods::scrExit(void)
 {
-        while(stack)
-                pull();
+	while(stack)
+		pull();
 
-        frame[stack].line = NULL;
-        return true;
+	frame[stack].line = NULL;
+	return true;
 }
 
 bool ScriptMethods::scrSignal(void)
@@ -759,7 +759,7 @@ bool ScriptMethods::scrSignal(void)
 	{
 		if(!signal(++cp))
 			error("signal-trap-invalid");
-		
+
 		return true;
 	}
 
@@ -825,7 +825,7 @@ bool ScriptMethods::scrRestart(void)
 	frame[stack].line = frame[stack].first = frame[stack].script->first;
 	frame[stack].index = 0;
 	if(isFunction(frame[stack].script))
-                frame[stack].tranflag = true;
+		frame[stack].tranflag = true;
 
 	return true;
 }
@@ -843,7 +843,7 @@ bool ScriptMethods::scrTimer(void)
 
 	while(NULL != (cp = getOption(NULL)))
 	{
-		sym = mapSymbol(cp, 23);		
+		sym = mapSymbol(cp, 23);
 		if(!sym)
 		{
 err:
@@ -884,7 +884,7 @@ bool ScriptMethods::scrCounter(void)
 
 	while(NULL != (cp = getOption(NULL)))
 	{
-		sym = mapSymbol(cp, 11);		
+		sym = mapSymbol(cp, 11);
 		if(!sym)
 		{
 			err = "var-symbol-invalid";
@@ -925,7 +925,7 @@ bool ScriptMethods::scrSession(void)
 	}
 	else
 		error("session-invalid-id");
-	
+
 	return true;
 }
 
@@ -974,6 +974,85 @@ bool ScriptMethods::scrLock(void)
 	return true;
 }
 
+bool ScriptMethods::scrDeconstruct(void)
+{
+	const char *cp = getOption(NULL);
+	const char *bp;
+	char *dp;
+	char name[65];
+	char value[960];
+	Symbol *sym = mapSymbol(cp, 0);
+	Symbol *dest;
+
+	if(!sym)
+	{
+		error("invalid-symbol");
+		return true;
+	}
+
+	bp = sym->data;
+	while(*bp)
+	{
+		dp = name;
+		while(*bp && *bp != ':' && *bp != '=')
+			*(dp++) = *(bp++);
+		*dp = 0;
+		if(!*bp)
+			break;
+		else
+			++bp;
+		dp = value;
+		while(*bp && *bp != ';')
+			*(dp++) = *(bp++);
+		*dp = 0;
+		if(*bp == ';')
+			++bp;
+		dest = getKeysymbol(name);
+		if(!dest)
+			continue;
+		commit(dest, value);
+	}
+	skip();
+	return true;
+}
+
+bool ScriptMethods::scrConstruct(void)
+{
+	const char *target = getOption(NULL);
+	unsigned idx = 0;
+	char buffer[1024];
+	unsigned count = 0;
+	const char *tag, *value;
+	Line *line = getLine();
+	char *ep;
+
+	++target;
+	while(idx < line->argc)
+	{
+		tag = line->args[idx++];
+		if(!tag || *tag != '=')
+			continue;
+
+		++tag;
+		value = getContent(line->args[idx++]);
+		if(!value)
+			continue;
+
+		if(count)
+			snprintf(buffer + count, sizeof(buffer) - count, ";%s=%s",
+				tag, value);
+		else
+			snprintf(buffer + count, sizeof(buffer) - count, "%s=%s",
+				tag, value);
+		while(NULL != (ep = strchr(buffer + count + 1, ';')))
+			*ep = ',';
+		count = strlen(buffer);
+	}
+	setConst(target, buffer);
+	advance();
+	return true;
+}
+
 bool ScriptMethods::scrPack(void)
 {
 	char pack = getPackToken();
@@ -1031,6 +1110,28 @@ bool ScriptMethods::scrPack(void)
 	tokbuf[0] = 0;
 	buffer[0] = 0;
 
+	if(!stricmp(line->cmd, "unpack.struct"))
+	{
+		ep = sym->data;
+		while(ep && *ep)
+		{
+			lp = strchr(ep, ':');
+			if(!lp)
+				lp = strchr(ep, '=');
+			if(!lp)
+				break;
+			*(lp++) = 0;
+			cp = (const char *)ep;
+			ep = strchr(lp, ';');
+			if(ep)
+				*(ep++) = 0;
+			setConst(cp, lp);
+		}
+		clear(sym);
+		advance();
+		return true;
+	}
+
 	if(!unpack && prefix)
 		setString(buffer, sizeof(buffer), prefix);
 
@@ -1056,7 +1157,7 @@ bool ScriptMethods::scrPack(void)
 			else if(*lp == '"')
 				q = "\"";
 		}
-	}			
+	}
 
 	while(unpack && NULL != (cp = getOption(NULL)))
 	{
@@ -1074,14 +1175,14 @@ bool ScriptMethods::scrPack(void)
 				return true;
 			}
 		}
-		
+
 		if(*lp == pack)
 		{
 			commit(sym, "");
 			++lp;
 			continue;
 		}
-		
+
 		if(q && !strnicmp(lp, q, strlen(q)))
 		{
 			lp += strlen(q);
@@ -1110,7 +1211,7 @@ bool ScriptMethods::scrPack(void)
 			break;
 		lp = ep;
 	}
-				
+
 	while(!unpack)
 	{
 		if(offset)
@@ -1119,7 +1220,7 @@ bool ScriptMethods::scrPack(void)
 			--offset;
 		}
 		else
-			cp = getValue(NULL);
+			cp = getOption(NULL);
 
 		if(!cp)
 			break;
@@ -1127,13 +1228,30 @@ bool ScriptMethods::scrPack(void)
 			addString(buffer, sizeof(buffer), tokbuf);
 		tokbuf[0] = pack;
 		tokbuf[1] = 0;
+		if(!stricmp(line->cmd, "pack.struct"))
+		{
+			tokbuf[0] = ';';
+			addString(buffer, sizeof(buffer), cp + 1);
+			addString(buffer, sizeof(buffer), ":");
+			lp = buffer + strlen(buffer);
+		}
+		else
+			lp = NULL;
 		if(q)
 			addString(buffer, sizeof(buffer), q);
-		addString(buffer, sizeof(buffer), cp);
+		cp = getContent(cp);
+		if(cp)
+			addString(buffer, sizeof(buffer), cp);
+		while(lp && *lp)
+		{
+			lp = strchr(lp, ';');
+			if(lp)
+				*(lp++) = ',';
+		}
 		if(q)
 			addString(buffer, sizeof(buffer), q);
 	}
-	
+
 	if(!unpack)
 	{
 		if(suffix)
@@ -1148,7 +1266,7 @@ bool ScriptMethods::scrPack(void)
 
 	skip();
 	return true;
-}		
+}
 
 bool ScriptMethods::scrClear(void)
 {
@@ -1228,10 +1346,10 @@ bool ScriptMethods::scrDefine(void)
 {
 	char var[128];
 	char base[65];
-        Symbol *sym;
-        const char *cp, *pv;
-        const char *prefix = getMember();
-        unsigned len;
+	Symbol *sym;
+	const char *cp, *pv;
+	const char *prefix = getMember();
+	unsigned len;
 	char *ep;
 	Line *line = getLine();
 	Name *scr = getName();
@@ -1257,18 +1375,18 @@ bool ScriptMethods::scrDefine(void)
 			if(*pv == '{')
 				++pv;
 		}
-		
+
 		if(prefix && !strchr(cp, '.') && *cp != '%' && *cp != '&')
 	                snprintf(var, sizeof(var), "%s.%s", prefix, cp);
 		else
 			setString(var, sizeof(var), cp);
 
-                ep = strrchr(var, ':');
-                if(ep)
-                {
-                        *(ep++) = 0;
-                        len = atoi(ep);
-                }
+		ep = strrchr(var, ':');
+		if(ep)
+		{
+			*(ep++) = 0;
+			len = atoi(ep);
+		}
 		else if(pv)
 			len = 0;
 		else
@@ -1285,7 +1403,7 @@ bool ScriptMethods::scrDefine(void)
 		if(!vlen)
 			++vlen;
 
-                sym = mapSymbol(var, vlen);
+		sym = mapSymbol(var, vlen);
 		if(!sym || sym->type != symINITIAL)
 			continue;
 
@@ -1340,7 +1458,7 @@ bool ScriptMethods::scrVar(void)
 	if(errmsg)
 		error(errmsg);
 	else
-		skip();	
+		skip();
 
 	return true;
 }
@@ -1384,7 +1502,7 @@ bool ScriptMethods::scrExpr(void)
 				prec = sym->size - 12;
 			continue;
 		}
-		
+
 		error("expr-invalid");
 		return true;
 	}
@@ -1402,7 +1520,7 @@ bool ScriptMethods::scrExpr(void)
 		error("expr-invalid");
 		return true;
 	}
-	
+
 	snprintf(fmt, sizeof(fmt), "%s%d%s", "%ld.%0", prec, "ld");
 	frame[stack].index = 0;
 	hNumber = iNumber / tens[prec];
@@ -1430,10 +1548,10 @@ bool ScriptMethods::scrExpr(void)
 			{
 				error("expr-cannot-assign");
 				return true;
-			}	
+			}
 			continue;
 		}
-		
+
 		a = (Array *)&sym->data;
 		switch(sym->type)
 		{
@@ -1454,7 +1572,7 @@ bool ScriptMethods::scrExpr(void)
 		default:
 			error("expr-cannot-assign");
 			return true;
-		}				
+		}
 	}
 	skip();
 	return true;
@@ -1491,7 +1609,7 @@ bool ScriptMethods::scrIndex(void)
 			}
 			continue;
 		}
-		
+
 		error("index-invalid");
 		return true;
 	}
@@ -1506,7 +1624,7 @@ bool ScriptMethods::scrIndex(void)
 		error("index-invalid");
 		return true;
 	}
-	
+
 	while(NULL != (opt = getOption(NULL)))
 	{
 		if(!stricmp(opt, "-eq"))
@@ -1543,7 +1661,7 @@ bool ScriptMethods::scrNumber(void)
 	cp = getKeyword("decimal");
 	if(cp)
 		size = 12 + atoi(cp);
-	
+
 	while(NULL != (cp = getOption(NULL)))
 	{
 		sym = mapSymbol(cp, size);
@@ -1566,7 +1684,7 @@ bool ScriptMethods::scrNumber(void)
 	if(errmsg)
 		error(errmsg);
 	else
-		skip();	
+		skip();
 
 	return true;
 }
@@ -1589,20 +1707,20 @@ bool ScriptMethods::scrRef(void)
 		error("ref-missing-source");
 		return true;
 	}
-	
+
 	syms = getSymbols(sym->id);
 	if(syms != dynamic_cast<ScriptSymbols*>(this) && syms != frame[stack].local)
 	{
 		error("ref-invalid-source");
 		return true;
 	}
-	
+
 	if(strchr(target, '.'))
 	{
 		error("ref-invalid-target");
 		return true;
 	}
-	
+
 	if(frame[stack].local)
 		syms = frame[stack].local;
 	else
@@ -1714,58 +1832,58 @@ retry:
 		}
 		return true;
 	}
-        len = (unsigned)strlen(label);
-        if(!strncmp(label, "::", 2))
-        {
+	len = (unsigned)strlen(label);
+	if(!strncmp(label, "::", 2))
+	{
 		setString(namebuf, sizeof(namebuf), frame[stack].script->name);
-                ext = strstr(namebuf, "::");
-                if(ext)
+		ext = strstr(namebuf, "::");
+		if(ext)
 			setString(ext, sizeof(namebuf) + namebuf - ext, label);
-                else
+		else
 			addString(namebuf, sizeof(namebuf), label);
-                label = namebuf;
-        }
-        else if(label[len - 1] == ':')
-        {
+		label = namebuf;
+	}
+	else if(label[len - 1] == ':')
+	{
 		setString(namebuf, sizeof(namebuf), frame[stack].script->name);
-                ext = strstr(namebuf, "::");
-                if(ext)
+		ext = strstr(namebuf, "::");
+		if(ext)
 			*ext = 0;
 
 		addString(namebuf, sizeof(namebuf), "::");
 		addString(namebuf, sizeof(namebuf), label);
 
-                label = ext = namebuf;
+		label = ext = namebuf;
 		len = (unsigned)strlen(label);
 		ext[len - 1] = 0;
-        }
+	}
 
-        scr = getScript(label);
-        if(!scr)
-        {
+	scr = getScript(label);
+	if(!scr)
+	{
 		label = getValue(NULL);
 		if(label)
 			goto retry;
-                error("script-not-found");
-                return true;
-        }
+		error("script-not-found");
+		return true;
+	}
 	clearStack();
 	frame[stack].caseflag = frame[stack].tranflag = false;
-        frame[stack].script = scr;
-        frame[stack].line = frame[stack].first = scr->first;
-        frame[stack].index = 0;
+	frame[stack].script = scr;
+	frame[stack].line = frame[stack].first = scr->first;
+	frame[stack].index = 0;
 	frame[stack].mask = getMask();
-        return true;
+	return true;
 }
 
 bool ScriptMethods::scrCall(void)
 {
-        Line *line = getLine();
+	Line *line = getLine();
 	const char *cmd = line->cmd;
 	const char *cp, *vp;
 	unsigned idx = 0;
 	Symbol *sym;
-	
+
 	if(!push())
 	{
 		error("stack-overflow");
@@ -1809,7 +1927,7 @@ bool ScriptMethods::scrCall(void)
 	}
 	return redirect(false);
 }
-	 
+
 bool ScriptMethods::scrBegin(void)
 {
 	if(frame[stack].tranflag)
@@ -1825,14 +1943,14 @@ bool ScriptMethods::scrBegin(void)
 
 bool ScriptMethods::scrEnd(void)
 {
-        if(!frame[stack].tranflag)
-        {
-                error("end-not-in-transaction");
-                return true;       
-        }
-        frame[stack].tranflag = false;
-        skip();
-        return true;
+	if(!frame[stack].tranflag)
+	{
+		error("end-not-in-transaction");
+		return true;
+	}
+	frame[stack].tranflag = false;
+	skip();
+	return true;
 }
 
 bool ScriptMethods::scrCase(void)
@@ -1877,9 +1995,9 @@ bool ScriptMethods::scrCase(void)
 
 bool ScriptMethods::scrEndcase(void)
 {
-        frame[stack].caseflag = frame[stack].tranflag = false;
-        skip();
-        return true;
+	frame[stack].caseflag = frame[stack].tranflag = false;
+	skip();
+	return true;
 }
 
 bool ScriptMethods::scrOffset(void)
@@ -1898,7 +2016,7 @@ bool ScriptMethods::scrOffset(void)
 
 	line = frame[stack - 1].line;
 	method = line->scr.method;
-	
+
 	if(method != (Method)&ScriptMethods::scrForeach &&
 	   method != (Method)&ScriptMethods::scrFor)
 	{
@@ -1921,65 +2039,65 @@ bool ScriptMethods::scrOffset(void)
 
 bool ScriptMethods::scrRepeat(void)
 {
-        unsigned short loop = frame[stack].line->loop;       
-        Line *line;
-        int index = frame[stack].index;
-        long count;
+	unsigned short loop = frame[stack].line->loop;
+	Line *line;
+	int index = frame[stack].index;
+	long count;
 
-        frame[stack].index = 0;
+	frame[stack].index = 0;
 	numericExpression(&count, 1, 0);
 
-        if(index >= count)
-        {
-                line = frame[stack].line->next;
-                while(line)
-                {
-                        if(line->loop == loop)
-                        {
-                                frame[stack].line = line;
-                                skip();
-                                return true;
-                        }
-                        line = line->next;
-                }
-                error("loop-overflow");
-                return true;
-        }
+	if(index >= count)
+	{
+		line = frame[stack].line->next;
+		while(line)
+		{
+			if(line->loop == loop)
+			{
+				frame[stack].line = line;
+				skip();
+				return true;
+			}
+			line = line->next;
+		}
+		error("loop-overflow");
+		return true;
+	}
 
-        frame[stack].index = ++index;
-        if(!push())
-                return true;
+	frame[stack].index = ++index;
+	if(!push())
+		return true;
 
-        skip();
-        return true;
+	skip();
+	return true;
 }
 
 bool ScriptMethods::scrFor(void)
 {
-        Symbol *sym;
-        unsigned short loop = frame[stack].line->loop;
-        Line *line;
-        int index = frame[stack].index;
-        unsigned size = symsize;
+	Symbol *sym;
+	unsigned short loop = frame[stack].line->loop;
+	Line *line;
+	int index = frame[stack].index;
+	unsigned size = symsize;
 	const char *opt;
-        const char *kw = getKeyword("size");
+	const char *kw = getKeyword("size");
 	char buffer[12];
 
-        if(kw)
-                size = atoi(kw);
+	if(kw)
+		size = atoi(kw);
 
-        frame[stack].index = 0;
+	frame[stack].index = 0;
 	opt = getOption(NULL);
 
-        if(!index)
-        {
+	if(!index)
+	{
 		kw = getKeyword("index");
-                if(kw)
-                        index = atoi(kw);
-        }
+		if(kw)
+			index = atoi(kw);
+	}
 
-        if(!index)
-                ++index;
+	if(!index)
+		++index;
 
 	sym = getKeysymbol("index");
 	if(sym)
@@ -1988,34 +2106,34 @@ bool ScriptMethods::scrFor(void)
 		commit(sym, buffer);
 	}
 
-        sym = mapSymbol(opt, size);
+	sym = mapSymbol(opt, size);
 
-        if(!sym)
-        {
-                error("symbol-not-found");
-                return true;
-        }
+	if(!sym)
+	{
+		error("symbol-not-found");
+		return true;
+	}
 
-        frame[stack].index = index;
-        opt = getValue(NULL);
+	frame[stack].index = index;
+	opt = getValue(NULL);
 
 	if(!opt)
 	{
 failed:
-                line = frame[stack].line->next;
-                while(line)
-                {
-                        if(line->loop == loop)
-                        {
-                                frame[stack].line = line;
-                                skip();
-                                return true;
-                        }
-                        line = line->next;
-                }
-                error("loop-overflow");
-                return true;
-        }
+		line = frame[stack].line->next;
+		while(line)
+		{
+			if(line->loop == loop)
+			{
+				frame[stack].line = line;
+				skip();
+				return true;
+			}
+			line = line->next;
+		}
+		error("loop-overflow");
+		return true;
+	}
 
 	if(!push())
 		goto failed;
@@ -2030,59 +2148,59 @@ failed:
 
 bool ScriptMethods::scrForeach(void)
 {
-        Symbol *sym, *src;
-        unsigned short loop = frame[stack].line->loop;
-        Line *line;
-        int index = frame[stack].index;
-        unsigned size = symsize;
+	Symbol *sym, *src;
+	unsigned short loop = frame[stack].line->loop;
+	Line *line;
+	int index = frame[stack].index;
+	unsigned size = symsize;
 	const char *opt, *val;
 	char pack = getPackToken();
-        const char *kw = getKeyword("size");
+	const char *kw = getKeyword("size");
 	char buffer[1024];
 	char *cp;
 	Array *a;
 	ScriptProperty *p;
 
-        if(kw)
-                size = atoi(kw);
+	if(kw)
+		size = atoi(kw);
 
 	kw = getKeyword("token");
 	if(kw && *kw)
 		pack = *kw;
 
-        frame[stack].index = 0;
+	frame[stack].index = 0;
 	opt = getOption(NULL);
 	val = getOption(NULL);
 
-        if(!index)
-        {
-                kw = getKeyoption("index");
+	if(!index)
+	{
+		kw = getKeyoption("index");
 		if(kw)
 			kw = getSymContent(kw);
-                if(kw)
-                        index = atoi(kw);
-        }
+		if(kw)
+			index = atoi(kw);
+	}
 
-        if(!index)
-                ++index;
+	if(!index)
+		++index;
 
 	sym = getKeysymbol("index");
 	if(sym)
 	{
 		snprintf(buffer, 11, "%d", index);
 		commit(sym, buffer);
-	}		
+	}
 
-        sym = mapSymbol(opt, size);
+	sym = mapSymbol(opt, size);
 	src = mapSymbol(val, 0);
 
-        if(!sym || !src)
-        {
-                error("symbol-not-found");
-                return true;
-        }
+	if(!sym || !src)
+	{
+		error("symbol-not-found");
+		return true;
+	}
 
-        frame[stack].index = index;
+	frame[stack].index = index;
 	++frame[stack].index;
 
 	opt = NULL;
@@ -2100,7 +2218,7 @@ bool ScriptMethods::scrForeach(void)
 	default:
 		break;
 	case symPROPERTY:
-                memcpy(&p, &sym->data, sizeof(p));
+		memcpy(&p, &sym->data, sizeof(p));
 		if(p->token())
 			pack = p->token();
 	case symNORMAL:
@@ -2119,26 +2237,26 @@ bool ScriptMethods::scrForeach(void)
 			cp = (char *)strchr(opt, pack);
 			if(cp)
 				*cp = 0;
-		}		
+		}
 	}
 
 	if(!opt)
 	{
 failed:
-                line = frame[stack].line->next;
-                while(line)
-                {
-                        if(line->loop == loop)
-                        {
-                                frame[stack].line = line;
-                                skip();
-                                return true;
-                        }
-                        line = line->next;
-                }
-                error("loop-overflow");
-                return true;
-        }
+		line = frame[stack].line->next;
+		while(line)
+		{
+			if(line->loop == loop)
+			{
+				frame[stack].line = line;
+				skip();
+				return true;
+			}
+			line = line->next;
+		}
+		error("loop-overflow");
+		return true;
+	}
 
 	if(!push())
 		goto failed;
