@@ -1,19 +1,19 @@
 // Copyright (C) 1999-2005 Open Source Telecom Corporation.
-//  
+//
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
 // the Free Software Foundation; either version 2 of the License, or
 // (at your option) any later version.
-// 
+//
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
 // You should have received a copy of the GNU General Public License
-// along with this program; if not, write to the Free Software 
+// along with this program; if not, write to the Free Software
 // Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
-// 
+//
 // As a special exception, you may use this file as part of a free software
 // library without restriction.  Specifically, if other files instantiate
 // templates or use macros or inline functions from this file, or you compile
@@ -56,6 +56,10 @@ ScriptCommand()
 			(Check)&ScriptChecks::chkPack},
 		{"pack", true, (Method)&ScriptMethods::scrPack,
 			(Check)&ScriptChecks::chkPack},
+		{"construct", true, (Method)&ScriptMethods::scrConstruct,
+			(Check)&ScriptChecks::chkConstruct},
+		{"deconstruct", true, (Method)&ScriptMethods::scrDeconstruct,
+			(Check)&ScriptChecks::chkConstruct},
 		{"slog", false, (Method)&ScriptMethods::scrSlog,
 			(Check)&ScriptChecks::chkSlog},
 		{"syslog", false, (Method)&ScriptMethods::scrSlog,
@@ -94,12 +98,12 @@ ScriptCommand()
 			(Check)&ScriptChecks::chkForeach},
 		{"do", false, (Method)&ScriptMethods::scrDo,
 			(Check)&ScriptChecks::chkConditional},
-                {"loop", false, (Method)&ScriptMethods::scrLoop,
-                        (Check)&ScriptChecks::chkConditional}, 
-                {"continue", false, (Method)&ScriptMethods::scrContinue,
-                        (Check)&ScriptChecks::chkConditional}, 
-                {"break", false, (Method)&ScriptMethods::scrBreak,
-                        (Check)&ScriptChecks::chkConditional}, 
+		{"loop", false, (Method)&ScriptMethods::scrLoop,
+			(Check)&ScriptChecks::chkConditional},
+		{"continue", false, (Method)&ScriptMethods::scrContinue,
+			(Check)&ScriptChecks::chkConditional},
+		{"break", false, (Method)&ScriptMethods::scrBreak,
+			(Check)&ScriptChecks::chkConditional},
 		{"error", false, (Method)&ScriptMethods::scrError,
 			(Check)&ScriptChecks::chkError},
 		{"decimal", true, (Method)&ScriptMethods::scrDecimal,
@@ -126,16 +130,16 @@ ScriptCommand()
 			(Check)&ScriptChecks::chkVar},
 		{"type", true, (Method)&ScriptMethods::scrType,
 			(Check)&ScriptChecks::chkVarType},
-                {"vartype", true, (Method)&ScriptMethods::scrType,
-                        (Check)&ScriptChecks::chkVarType},
+		{"vartype", true, (Method)&ScriptMethods::scrType,
+			(Check)&ScriptChecks::chkVarType},
 		{"define", true, (Method)&ScriptMethods::scrDefine,
 			(Check)&ScriptChecks::chkDefine},
-                {"symbols", true, (Method)&ScriptMethods::scrDefine,
-                        (Check)&ScriptChecks::chkDefine},
+		{"symbols", true, (Method)&ScriptMethods::scrDefine,
+			(Check)&ScriptChecks::chkDefine},
   		{"string", true, (Method)&ScriptMethods::scrVar,
-			(Check)&ScriptChecks::chkString}, 
-                {"str", true, (Method)&ScriptMethods::scrVar,     
-                        (Check)&ScriptChecks::chkString},
+			(Check)&ScriptChecks::chkString},
+		{"str", true, (Method)&ScriptMethods::scrVar,
+			(Check)&ScriptChecks::chkString},
 		{"char", true, (Method)&ScriptMethods::scrVar,
 			(Check)&ScriptChecks::chkChar},
 		{"signal", false, (Method)&ScriptMethods::scrSignal,
@@ -154,7 +158,7 @@ ScriptCommand()
 			(Check)&ScriptChecks::chkReturn},
 		{"restart", false, (Method)&ScriptMethods::scrRestart,
 			(Check)&ScriptChecks::chkRestart},
-		{"ref", true,  (Method)&ScriptMethods::scrRef, 
+		{"ref", true,  (Method)&ScriptMethods::scrRef,
 			(Check)&ScriptChecks::chkRefArgs},
 		{"fconst", true, (Method)&ScriptMethods::scrInit,
 			(Check)&ScriptChecks::chkIgnore},
@@ -162,18 +166,18 @@ ScriptCommand()
 			(Check)&ScriptChecks::chkIgnore},
 		{"keywords", false, (Method)&ScriptMethods::scrNop,
 			(Check)&ScriptChecks::chkKeywords},
-                {"keyword", false, (Method)&ScriptMethods::scrNop,
-                        (Check)&ScriptChecks::chkKeywords},
+		{"keyword", false, (Method)&ScriptMethods::scrNop,
+			(Check)&ScriptChecks::chkKeywords},
 		{"sequence", true, (Method)&ScriptMethods::scrSequence,
 			(Check)&ScriptChecks::chkSequence},
 		{"const", true, (Method)&ScriptMethods::scrConst,
 			(Check)&ScriptChecks::chkConst},
-                {"stack", true, (Method)&ScriptMethods::scrArray,
-                        (Check)&ScriptChecks::chkArray},
-                {"fifo", true, (Method)&ScriptMethods::scrArray,
-                        (Check)&ScriptChecks::chkArray},
-                {"lifo", true, (Method)&ScriptMethods::scrArray,
-                        (Check)&ScriptChecks::chkArray},
+		{"stack", true, (Method)&ScriptMethods::scrArray,
+			(Check)&ScriptChecks::chkArray},
+		{"fifo", true, (Method)&ScriptMethods::scrArray,
+			(Check)&ScriptChecks::chkArray},
+		{"lifo", true, (Method)&ScriptMethods::scrArray,
+			(Check)&ScriptChecks::chkArray},
 		{"array", true, (Method)&ScriptMethods::scrArray,
 			(Check)&ScriptChecks::chkArray},
 		{"init", true, (Method)&ScriptMethods::scrSet,
@@ -194,11 +198,11 @@ ScriptCommand()
 			(Check)&ScriptChecks::chkSession},
 		{NULL, false, NULL, NULL}};
 
-        // add special default exit and error traps
+	// add special default exit and error traps
 
-        trap("exit");
-        trap("error", false);
-        load(interp);
+	trap("exit");
+	trap("error", false);
+	load(interp);
 
 	// add common module aliasing
 #ifdef	SQL_MODULE
