@@ -1,19 +1,19 @@
 // Copyright (C) 1999-2005 Open Source Telecom Corporation.
-//  
+//
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
 // the Free Software Foundation; either version 2 of the License, or
 // (at your option) any later version.
-// 
+//
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
 // You should have received a copy of the GNU General Public License
-// along with this program; if not, write to the Free Software 
+// along with this program; if not, write to the Free Software
 // Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
-// 
+//
 // As a special exception, you may use this file as part of a free software
 // library without restriction.  Specifically, if other files instantiate
 // templates or use macros or inline functions from this file, or you compile
@@ -76,7 +76,7 @@
 #elif !defined(__hpux) && !defined(_AIX)
 #define CCXX_PACKED
 #endif
-#endif  
+#endif
 
 namespace ost {
 
@@ -120,7 +120,7 @@ public:
 	};
 
 	typedef enum scrAccess scrAccess;
-		
+
 	enum symType
 	{
 		symNORMAL = 0,
@@ -205,30 +205,30 @@ public:
 		scrAccess access;
 	};
 
-        class __EXPORT Initial
-        {
-        public:
-                const char *name;
-                unsigned size;
-                const char *value;
-        };
+	class __EXPORT Initial
+	{
+		public:
+		const char *name;
+		unsigned size;
+		const char *value;
+	};
 
-        class __EXPORT Define
-        {
-        public:
-                const char *keyword;
+	class __EXPORT Define
+	{
+		public:
+		const char *keyword;
 		bool init;
-                Method method;   
-                Check check; 
-        };
+		Method method;
+		Check check;
+	};
 
-        class __EXPORT Test
-        {
-        public:
-                const char *id;
-                Cond handler;
-                Test *next;  
-        };
+	class __EXPORT Test
+	{
+		public:
+		const char *id;
+		Cond handler;
+		Test *next;
+	};
 
 	class __EXPORT Fun
 	{
@@ -244,7 +244,7 @@ public:
 	public:
 		Init handler;
 		InitScript *next;
-	};	
+	};
 
 	class __EXPORT Package : public DSO
 	{
@@ -252,7 +252,7 @@ public:
 		static Package *first;
 		Package *next;
 		const char *filename;
-	
+
 		Package(const char *name);
 	};
 
@@ -299,16 +299,16 @@ public:
 	static const char *var_prefix;
 	static const char *log_prefix;
 
-        static void addFunction(const char *name, unsigned count, Function i);
-        static void addConditional(const char *name, Cond test);
+	static void addFunction(const char *name, unsigned count, Function i);
+	static void addConditional(const char *name, Cond test);
 
 	static bool isPrivate(Name *scr);
 	static bool isFunction(Name *scr);
 
-protected:        
-        static Test *test;
-        static Fun *ifun;
-        
+protected:
+	static Test *test;
+	static Fun *ifun;
+
 };
 
 class __EXPORT ScriptSymbols : public MemPager, public Script
@@ -478,7 +478,7 @@ protected:
 	 * @return script object of trap mask to use.
 	 */
 	virtual unsigned long getTrapHandler(Name *script);
-  
+
 	/**
 	 * Get a trap mask for a given identifer.  This is a virtual
 	 * since some derived handlers may manipulate mask bits.
@@ -526,15 +526,15 @@ public:
 	 */
 	static const char *findKeyword(Line *line, const char *keyword);
 
-    /**
-     * Test for a specific keyword or keydata filler.
-     *
-     * @return content of keyword that is found.
+	/**
+	 * Test for a specific keyword or keydata filler.
+	 *
+	 * @return content of keyword that is found.
 	 * @param script image for keydata.
-     * @param line pointer to record to examine in check routine.
-     * @param keyword to search for.
-     */
-    static const char *findKeyword(ScriptImage *img, Line *line, const char *keyword);
+	 * @param line pointer to record to examine in check routine.
+	 * @param keyword to search for.
+	 */
+	static const char *findKeyword(ScriptImage *img, Line *line, const char *keyword);
 
 	/**
 	 * Server level logging interface override.
@@ -667,7 +667,7 @@ private:
 
 protected:
 	void bind(Script::Define *extensions);
-	
+
 	virtual void attach(ScriptInterp *interp);
 	virtual void detach(ScriptInterp *interp);
 	virtual bool select(ScriptInterp *interp);
@@ -695,7 +695,7 @@ public:
 class __EXPORT ScriptRipple : public ScriptCommand
 {
 public:
-        ScriptRipple();
+	ScriptRipple();
 };
 
 class __EXPORT ScriptChecks : public ScriptCommand
@@ -752,9 +752,9 @@ public:
 	 * statements such as "clear".  Assumes list of valid variable
 	 * arguments.
 	 *
-         * @return syntax error message string or NULL.
-         * @param line statement being examined.
-         * @param img pointer to image being compiled.
+	 * @return syntax error message string or NULL.
+	 * @param line statement being examined.
+	 * @param img pointer to image being compiled.
 	 */
 	const char *chkHasVars(Line *line, ScriptImage *img);
 
@@ -802,9 +802,9 @@ public:
 	 * Script compiler syntax check for commands that require
 	 * one or more arguments but use no keywords.
 	 *
-         * @return syntax error message string or NULL.
-         * @param line statement being examined.
-         * @param img pointer to image being compiled.
+	 * @return syntax error message string or NULL.
+	 * @param line statement being examined.
+	 * @param img pointer to image being compiled.
 	 */
 	const char *chkOnlyArgs(Line *line, ScriptImage *img);
 
@@ -878,6 +878,8 @@ public:
 
 	const char *chkPack(Line *line, ScriptImage *img);
 
+	const char *chkConstruct(Line *line, ScriptImage *img);
+
 	const char *chkLock(Line *line, ScriptImage *img);
 
 	const char *chkSession(Line *line, ScriptImage *img);
@@ -928,7 +930,7 @@ protected:
 	ScriptObject *objects;
 	static unsigned long serial;
 	unsigned long instance;
-	
+
 	class	__EXPORT InitialList : public Script::Initial
 	{
 	public:
@@ -1049,7 +1051,7 @@ public:
 
 	/**
 	 * Used in the derived constructor to "commit" the current image
-         * for new processes.  This is usually the last statement in the
+	 * for new processes.  This is usually the last statement in the
 	 * derived constructor.
 	 */
 	void commit(void);
@@ -1164,7 +1166,7 @@ protected:
 	}	merge_t;
 
 	merge_t *mlist;
-	
+
 	friend class __EXPORT ScriptInterp;
 
 	virtual bool checkSegment(Name *scr);
@@ -1237,7 +1239,7 @@ public:
 
 	/**
 	 * Used in the derived constructor to "commit" the current image
-         * for new processes.  This is usually the last statement in the
+	 * for new processes.  This is usually the last statement in the
 	 * derived constructor.
 	 */
 	void commit(void);
@@ -1291,16 +1293,16 @@ public:
 		unsigned decimal : 4;
 		unsigned base : 8;
 	};
-	
-        static long getRealValue(double val, unsigned prec);
-        static double getDouble(long value, unsigned prec);
-        static long getInteger(long value, unsigned prec);
-        static long getTens(unsigned prec);
-        long getIntValue(const char *text, unsigned prec, ScriptProperty *property = NULL);
-        int numericExpression(long *list, int max, unsigned prec, ScriptProperty *property = NULL);
+
+	static long getRealValue(double val, unsigned prec);
+	static double getDouble(long value, unsigned prec);
+	static long getInteger(long value, unsigned prec);
+	static long getTens(unsigned prec);
+	long getIntValue(const char *text, unsigned prec, ScriptProperty *property = NULL);
+	int numericExpression(long *list, int max, unsigned prec, ScriptProperty *property = NULL);
 	bool conditionalExpression(void);
 	bool conditional(void);
-	
+
 protected:
 	Mutex *lock;	// any additional lock that is siezed
 	ScriptCommand *cmd;
@@ -1315,7 +1317,7 @@ protected:
 	unsigned long sequence;
 	char logname[32];
 
-public:	
+public:
 	virtual unsigned getId(void);
 
 	inline unsigned long getSequence(void)
@@ -1350,16 +1352,16 @@ public:
 	char getPackToken(void);
 
 protected:
-        /**
-         * Initialize execution environment for a script.
-         */
-        void initRuntime(Name *name); 
+	/**
+	 * Initialize execution environment for a script.
+	 */
+	void initRuntime(Name *name);
 
-        /**
-         * New virtual to initialize script environment syms     
-         * before running init sections.
-         */
-        virtual void initialize(void);
+	/**
+	 * New virtual to initialize script environment syms
+	 * before running init sections.
+	 */
+	virtual void initialize(void);
 
 public:
 	inline Frame *getFrame(void)
@@ -1376,64 +1378,64 @@ public:
 	inline bool getTrace(void)
 		{return trace;};
 
-        /**
-         * Runtime execution of script handler.  This can be called in
-         * the current or derived class to invoke extensible methods.
-         *
-         * @return true if immediately ready for next step.      
-         * @param method derived method member to call.
-         */
-        bool execute(Method method);
+	/**
+	 * Runtime execution of script handler.  This can be called in
+	 * the current or derived class to invoke extensible methods.
+	 *
+	 * @return true if immediately ready for next step.
+	 * @param method derived method member to call.
+	 */
+	bool execute(Method method);
 
 protected:
-        /**
-         * Attempt to push a value onto the stack.
-         *
-         * @return false if stack overflow.
-         */
-        bool push(void);
+	/**
+	 * Attempt to push a value onto the stack.
+	 *
+	 * @return false if stack overflow.
+	 */
+	bool push(void);
 
-        /**
-         * Attempt to recall a previous stack level.
-         *
-         * @return false if stack underflow.
-         */
-        bool pull(void);
+	/**
+	 * Attempt to recall a previous stack level.
+	 *
+	 * @return false if stack underflow.
+	 */
+	bool pull(void);
 
-        /**
-         * Clear the stack of local loops or recursion for branching.
-         */
-        void clearStack(void);
+	/**
+	 * Clear the stack of local loops or recursion for branching.
+	 */
+	void clearStack(void);
 
-        /**
-         * Advance program to the next script statement.
-         */
-        void advance(void);    
+	/**
+	 * Advance program to the next script statement.
+	 */
+	void advance(void);
 
 	/**
 	 * Skip line without checking or setting updates.
 	 */
 	void skip(void);
 
-        /**
-         * Set error variable and advance to either the error handler     
-         * or next script statement.
-         *
-         * @param error message.
-         */
-        void error(const char *error);  
+	/**
+	 * Set error variable and advance to either the error handler
+	 * or next script statement.
+	 *
+	 * @param error message.
+	 */
+	void error(const char *error);
 
-        /**
-         * Events reference to named \@event handlers which have been
-         * attached to a script.  This allows low level applications
-         * to invoke an event handler much the way a signal handler
-         * occurs.
-         *
-         * @return true if event handler exists.
-         * @param name of event handler.
-         * @param inhereted search flag.
-         */
-        bool scriptEvent(const char *name, bool inhereted = true);
+	/**
+	 * Events reference to named \@event handlers which have been
+	 * attached to a script.  This allows low level applications
+	 * to invoke an event handler much the way a signal handler
+	 * occurs.
+	 *
+	 * @return true if event handler exists.
+	 * @param name of event handler.
+	 * @param inhereted search flag.
+	 */
+	bool scriptEvent(const char *name, bool inhereted = true);
 
 	/**
 	 * Branch to a selected event record immediately.
@@ -1442,14 +1444,14 @@ protected:
 	 */
 	void gotoEvent(NamedEvent *event);
 
-        /**
-         * Set the execution interpreter to a trap identifier.  If no
-         * trap id exists, then advance to next script statement (unless
-         * exit trap).
-         * 
-         * @param id of trap to select numerically.     
-         */
-        void trap(unsigned id);
+	/**
+	 * Set the execution interpreter to a trap identifier.  If no
+	 * trap id exists, then advance to next script statement (unless
+	 * exit trap).
+	 *
+	 * @param id of trap to select numerically.
+	 */
+	void trap(unsigned id);
 
 	/**
 	 * Tries a catch handler...
@@ -1459,13 +1461,13 @@ protected:
 	 */
 	bool tryCatch(const char *id);
 
-        /**
-         * Select trap by symbolic name and execute if found, else advance
-         * to next script step (unless exit trap).
-         *
-         * @param name of trap to select.
-         */
-        void trap(const char *name);
+	/**
+	 * Select trap by symbolic name and execute if found, else advance
+	 * to next script step (unless exit trap).
+	 *
+	 * @param name of trap to select.
+	 */
+	void trap(const char *name);
 
 public:
 	virtual void logmissing(const char *id, const char *level = "undefined", const char *group = "symbol");
@@ -1484,7 +1486,7 @@ protected:
 	virtual void startThread(void);
 
 	bool eventThread(const char *evt, bool flag = true);
-	
+
 	bool redirect(const char *scr);
 
 	void ripple(void);
@@ -1516,26 +1518,26 @@ public:
 	 * Release any aquired lock...
 	 */
 	void release(void);
-	
-       /**
-         * Signals are used during "delayed" execution steps when a    
-         * signal event has occured aynchronously with the execution
-         * of a script controlled state event handler.  This mechanism
-         * can be used in place of calling implicit "Step" traps.
-         *
-         * @return true if signal handler is not blocked.
-         * @param name of signal identifier.
-         */
-        bool signal(const char *name);
 
-        /**
-         * Signals can be referenced by numeric id as well as by symbolic
-         * name.
-         *
-         * @return true if signal handler is not blocked.
-         * @param id number of handler.
-         */
-        bool signal(unsigned id);
+	   /**
+	 * Signals are used during "delayed" execution steps when a
+	 * signal event has occured aynchronously with the execution
+	 * of a script controlled state event handler.  This mechanism
+	 * can be used in place of calling implicit "Step" traps.
+	 *
+	 * @return true if signal handler is not blocked.
+	 * @param name of signal identifier.
+	 */
+	bool signal(const char *name);
+
+	/**
+	 * Signals can be referenced by numeric id as well as by symbolic
+	 * name.
+	 *
+	 * @return true if signal handler is not blocked.
+	 * @param id number of handler.
+	 */
+	bool signal(unsigned id);
 
 	bool done(void);
 
@@ -1588,6 +1590,8 @@ public:
 	bool scrSet(void);
 	bool scrArray(void);
 	bool scrClear(void);
+	bool scrConstruct(void);
+	bool scrDeconstruct(void);
 	bool scrPack(void);
 	bool scrUnpack(void);
 	bool scrLock(void);
@@ -1624,7 +1628,7 @@ protected:
 	friend class __EXPORT ScriptInterp;
 
 	ScriptInterp *interp;
-	
+
 	void exit(const char *errmsg = NULL);
 
 	void exitEvent(const char *evt, bool inherited = true);
@@ -1651,7 +1655,7 @@ protected:
 
 public:
 	virtual timeout_t getTimeout(void);
-	
+
 	inline size_t getStack(void)
 		{return stacksize;};
 
@@ -1678,10 +1682,10 @@ public:
 
 /**
  * This class is used for DSO modules that impliment property
- * extensions for scripting objects.       
+ * extensions for scripting objects.
  *
  * @author David Sugar <dyfet@ostel.com>
- * @short ccScript property module  
+ * @short ccScript property module
  */
 class __EXPORT ScriptProperty : public Script
 {
@@ -1693,13 +1697,13 @@ private:
 	const char *id;
 
 public:
-        /**
-         * Set property method.  Performs set.xxx and init.xxx methods.
-         *
-         * @param data buffer to work from.
-         * @param temp workspace buffer to use.
-         * @param size of temp area.
-         */
+	/**
+	 * Set property method.  Performs set.xxx and init.xxx methods.
+	 *
+	 * @param data buffer to work from.
+	 * @param temp workspace buffer to use.
+	 * @param size of temp area.
+	 */
 	virtual void set(const char *data, char *temp, unsigned size) = 0;
 
 	/**
@@ -1709,22 +1713,22 @@ public:
 	 */
 	virtual unsigned prec(void);
 
-        /**
-         * Set property from integer value.
-         *
-         * @param data to save.
-         * @param size of data.
-         * @param value being set.
-         */
+	/**
+	 * Set property from integer value.
+	 *
+	 * @param data to save.
+	 * @param size of data.
+	 * @param value being set.
+	 */
 	virtual void setValue(char *data, unsigned short size, long value);
 
-        /**
-         * See if should be computed as property.
-         *
-         * @return true if property valid.
-         * @param data string to test.
-         */
-        virtual bool isProperty(const char *data);
+	/**
+	 * See if should be computed as property.
+	 *
+	 * @return true if property valid.
+	 * @param data string to test.
+	 */
+	virtual bool isProperty(const char *data);
 
 	/**
 	 * Initialize a new property through var definition.
@@ -1742,30 +1746,30 @@ public:
 	 */
 	virtual char token(void);
 
-        /**
-         * adjust value method.  Performs inc.xxx conversions.
-         *
-         * @param data buffer to work from.
-         * @param size of data buffer.
-         * @param adjustment offset to apply.
-         */
-        virtual void adjust(char *data, size_t size, long adjustment);
+	/**
+	 * adjust value method.  Performs inc.xxx conversions.
+	 *
+	 * @param data buffer to work from.
+	 * @param size of data buffer.
+	 * @param adjustment offset to apply.
+	 */
+	virtual void adjust(char *data, size_t size, long adjustment);
 
-        /**
-         * normalize values for scope and range.
-         *
-         * @return noramized value.
-         * @param value prior to normalization.
-         */
+	/**
+	 * normalize values for scope and range.
+	 *
+	 * @return noramized value.
+	 * @param value prior to normalization.
+	 */
 	virtual long adjustValue(long value);
 
-        /**
-         * Get the "numeric" (or #var) value of this property symbol.
-         *
-         * @return numeric value of this property object.
+	/**
+	 * Get the "numeric" (or #var) value of this property symbol.
+	 *
+	 * @return numeric value of this property object.
 	 * @param data being examined from property object.
-         */
-        virtual long getValue(const char *data);
+	 */
+	virtual long getValue(const char *data);
 
 	ScriptProperty(const char *name);
 	virtual ~ScriptProperty();
